import os
import sqlite3
import shutil


class UploadDb():
    def __init__(self, datasetId):
        self.datasetId = datasetId

        home = "USERPROFILE" if os.name == "nt" else "HOME"
        self.configDir = os.path.join(os.environ[home], ".tda/upload")
        if not os.path.exists(self.configDir):
            os.makedirs(self.configDir)

        self.dbFile = os.path.join(self.configDir, self.datasetId + ".db")
        if not os.path.exists(self.dbFile):
            self.conn = sqlite3.connect(self.dbFile)
            self.cur = self.conn.cursor()
            createSql = '''CREATE TABLE UPLOAD 
                              (
                                num INTEGER PRIMARY KEY,
                                filepath CHAR(1024)  NOT NULL,
                                object CHAR(1024) NOT NULL
                               );'''
            self.cur.execute(createSql)
            self.conn.close()

    def close(self):
        self.conn.close()

    def insertVal(self, filepath="", osspath=""):
        if filepath != "" and osspath != "":
            self.conn = sqlite3.connect(self.dbFile)
            self.cur = self.conn.cursor()
            self.cur.execute("INSERT INTO UPLOAD (num, filepath, object) values (?, ?, ?)", (None, filepath, osspath))
            self.conn.commit()
            self.conn.close()

    def findByFilePath(self, filepath="", osspath=""):
        if filepath != "" and osspath != "":
            self.conn = sqlite3.connect(self.dbFile)
            self.cur = self.conn.cursor()
            res = self.cur.execute(f"SELECT * FROM UPLOAD WHERE filepath = '{filepath}' and object = '{osspath}'")
            if len(res.fetchall()) >= 1:
                return True

        return False





import click
import json
import os
import requests
import testindatadev

from testindatadev.TDA import TDA
from testindatadev.utils import util

def _config_filepath():
    home = "USERPROFILE" if os.name == "nt" else "HOME"
    configDir = os.path.join(os.environ[home], ".tda/")
    if not os.path.exists(configDir):
        os.makedirs(configDir)

    return os.path.join(configDir, "tda.conf")


def _noLoginMessage():
    click.echo(" you have not set your account yet, to set your account with:\n")
    click.echo("  tda config <access_key> [host] \n")
    click.echo(" more about this command, use：\n")
    click.echo("  tda config --help")
    exit()


def _check():
    configFile = _config_filepath()
    if not os.path.exists(configFile):
        _noLoginMessage()

    with open(configFile, "r") as config:
        try:
            jsonObj = json.load(config)
        except:
            _noLoginMessage()

        if "access_key" not in jsonObj.keys():
            _noLoginMessage()

        if jsonObj["access_key"] == None or jsonObj["access_key"] == "":
            _noLoginMessage()

    return True


def _getConf():
    configFile = _config_filepath()
    if _check():
        with open(configFile, "r") as config:
            return json.load(config)

def _download(url, savePath):
    res = requests.get(url)
    with open(savePath, 'wb') as f:
        f.write(res.content)

def printVersion(ctx, param, value):
    if hasattr(testindatadev, "VERSION"):
        click.echo(f'TDA Version {testindatadev.VERSION}')
        ctx.exit()
    click.echo('TDA Version 0.0.1')
    ctx.exit()

@click.group()
@click.option("-ak", "--accessKey", 'access_key', default="", help="access key to Testin dataset system, see: https://dataset.testin.cn/accesskey")
@click.option("-host", "--host", 'host', default="https://dataset.testin.cn/", help="domain name that access to dataset system you would like to operate, default will be: https://dataset.testin.cn/")
@click.option('--version', is_flag=True, callback=printVersion, expose_value=False, is_eager=True)
@click.option('--debug/--no-debug', default=False, help="using debug mod or not, default is False")
@click.pass_context
def main(ctx, access_key, host, version, debug):
    """ testin dataset system management tool  """
    info = {
        "access_key": access_key,
        "host": host,
        "DEBUG": debug
    }

    _tda = None

    if access_key == "":
        configFile = _config_filepath()
        if os.path.exists(configFile):
            with open(configFile, "r", encoding="utf-8") as cf:
                if cf.read() != "":
                    info = _getConf()
                    info["DEBUG"] = debug

    if info["access_key"] != "":
        _tda = TDA(info["access_key"], info["host"])
        if info["DEBUG"]:
            _tda.Debug()

    ctx.obj = _tda



@main.command()
@click.option("-ak", "--accessKey", 'access_key', default="", help="access key to Testin dataset system, see: https://dataset.testin.cn/accesskey")
@click.option("-host", "--host", 'host', default="https://dataset.testin.cn/", help="domain name that access to dataset system you would like to operate, default will be: https://dataset.testin.cn/")
def config(access_key, host):
    """ setting your account """
    configFile = _config_filepath()
    if access_key == "":
        if _check():
            click.echo("logged user:")
            print(_getConf())
            exit()
    else:
        conf = {
            "access_key": access_key,
            "host": host,
        }
        with open(configFile, "w") as config:
            json.dump(conf, config)
            click.echo("login success")
            print(conf)
            exit()

@main.command()
@click.option("-ds", "--datasetId", 'ds_id', default="", help="the dataset you wolud like to download")
@click.option("-save", "--saveDir", 'save_dir', default="", help="save path for downloaded files")
@click.pass_context
def download(ctx, ds_id, save_dir):
    """ download dataset data """
    if ctx.obj == None:
        _noLoginMessage()

    ctx.obj.SetDataset(ds_id)
    saveDir = os.path.join(save_dir, ds_id)
    if not os.path.exists(saveDir):
        os.makedirs(saveDir)

    page = 0
    limit = 100
    fileTotal = 1
    while True:
        offset = page * limit
        fileData = ctx.obj.GetData(offset, limit)
        if len(fileData["files"]) <= 0:
            break
        page += 1
        for file in fileData["files"]:
            picPath = file.path.split(ds_id)[1].strip("/")
            basename = os.path.basename(picPath)
            tmpPath = picPath.replace(basename, "").strip("/")
            fileDir = os.path.join(saveDir, tmpPath)
            if not os.path.exists(fileDir):
                os.makedirs(fileDir)
            filePath = os.path.join(fileDir, tmpPath, basename)
            if os.path.exists(filePath):
                fmd5 = util.getFileMd5(filePath)
                if fmd5 != file.md5:
                    _download(file.url, filePath)
                    if ctx.obj.debug:
                        print(f"[SAVE_FILE]file total: {fileTotal}, truncate file, redo: [{filePath}]")
                else:
                    if ctx.obj.debug:
                        print(f"[SAVE_FILE]file total: {fileTotal}, file already exist: [{filePath}]")
            else:
                _download(file.url, filePath)
                if ctx.obj.debug:
                    print(f"[SAVE_FILE]file total: {fileTotal}, file download and save: [{filePath}]")

            labelData = ctx.obj.GetFileAndLabel(fid=file.fid)
            jsonname = ".".join(basename.split(".")[:-1])
            jsonPath = os.path.join(fileDir, tmpPath, jsonname + "_label.json")
            with open(jsonPath, "w", encoding="utf-8") as jf:
                json.dump(labelData.anotations.labels, jf)
                if ctx.obj.debug:
                    print(f"[SAVE_LABEL]file total: {fileTotal}, save label data: [{jsonPath}]")

            fileTotal += 1

    if ctx.obj.debug:
        print("done!")

if __name__ == '__main__':
    main(obj={})






import json

class LabelData():
    def __init__(self):
        self.labels = []

    def AddLabels(self, label="", instance="", attrs={}, type="", data={}, index=""):
        if label == "":
            raise Exception(f"label can not be empty!")

        label = {
            "label":label,
            "instance":instance,
            "type":type,
            "attrs":attrs,
            "data":data,
        }

        if index:
            label["index"] = index

        self.labels.append(label)

    def ToString(self):
        return json.dumps(self.labels)

    def LoadFromList(self, jsonData):
        if not type(jsonData) is list:
            raise Exception(f"Labeldata load from list error, {type(jsonData)} gavin")

        for data in jsonData:
            keys = list(data.keys())
            keys.sort()
            if keys != ['ano_id', 'attrs', 'data', 'index', 'instance', 'label', 'type'] and keys != ['ano_id', 'attrs', 'data', 'instance', 'label', 'type']:
                raise Exception(f"Labeldata load from list error, label data keys must be ['ano_id', 'attrs', 'data', 'index', 'instance', 'label', 'type'] or ['ano_id', 'attrs', 'data', 'instance', 'label', 'type']")

        self.labels = jsonData






from minio import Minio
from minio.error import InvalidResponseError as ResponseError
from datetime import timedelta
import requests

#约定：s3包下面的所有文件均是对存储对象的操作，里面所有的变量，单词，描述，均是和存储对象相关的，不允许出现dataset相关的用语
class YcMinio():
    def __init__(self, req = None):
        self.upUrl = ""
        self.req = req

    def PutObject(self, bucketName, objectName, filePath):
        urlInfo = self.req.GetUploadUrl(objectName)
        upUrl = urlInfo["url"]
        with open(filePath, 'rb') as file:
            try:
                response = requests.put(upUrl, data=file)
            except ResponseError as err:
                raise Exception("failed in deleting file", err)



import os
import sys
import json
import time
import re

from testindatadev.dataset.dataset import Dataset
from testindatadev.dataset.metadata import MetaData
from testindatadev.dataset.file import File
from testindatadev.dataset.urlfile import UrlFile
from testindatadev.dataset.commitdb import CommitDb
from testindatadev.dataset.uploaddb import UploadDb
from testindatadev.utils import util

#sdk entrance
class TDA():
    def __init__(self, accessKey, host="", debug=False):
        if host == "":
            self.host = "https://dataset.testin.cn/"
        else:
            if "http://" in host or "https://" in host:
                self.host = host.rstrip("/") + "/"
            else:
                self.host = "http://" + host.strip("/") + "/"
        self.__debug = debug
        self.T_key = accessKey
        self.commitFlag = False
        self.commitId = ""
        self.datasetId = ""
        self.fileList = []

    def Debug(self):
        """
        set sdk to debug mod
        """
        self.__debug = True

    @property
    def debug(self):
        '''
        debug read only
        :return:
        '''
        return self.__debug

    def SetDataset(self, datasetId):
        '''
        Set the dataset to be processed
        :param datasetId: the id of the dataset
        :return:
        '''
        self.datasetId = datasetId

        if self.debug:
            print(f"\033[0;37;44m[SET_DATASET] dataset: {self.datasetId} , service: {self.host}\033[0m")

        self.dataset = Dataset(self.T_key, datasetId, self.host)

        self.filePrefix = self.datasetId
        if self.dataset.file_path != "":
            self.filePrefix = self.dataset.file_path.strip("/") + "/" + self.datasetId.strip("/")

        if self.dataset.oss_type not in [self.dataset.OSS_TYPE_ALI, self.dataset.OSS_TYPE_QINIU]:
            self.filePrefix = self.dataset.bucket + "/" + self.filePrefix

        # return self.dataset#close for new feature

    def RiseException(self):
        raise Exception("You have not set the dataset name yet!")

    def UploadFilesToDataset(self, rootPath, ext=""):
        '''
        Upload files to dataset service directly
        :param rootPath: local file root path
        :param ext: if set, files endswith this extension will be uploaded
        :return:
        '''
        if self.debug:
            print(f"[UPLOAD_FILES] upload file to dataset without metedata and annotations!")

        if not self.datasetId:
            self.RiseException()

        total = 0
        for root, dir, files in os.walk(rootPath):
            for fileName in files:
                if fileName.endswith(ext):
                    total += 1

        if self.debug:
            print(f"[UPLOAD_FILES] file total: {total}")

        syncData = []
        j = 0
        i = 0
        retInfo = {'succ': 0, 'fail': 0}
        dirname = os.path.dirname(rootPath.rstrhost("/"))

        for root, dir, files in os.walk(rootPath):
            for fileName in files:
                if fileName.endswith(ext):
                    filePath = os.path.join(root, fileName)
                    objectName = filePath.replace(dirname, "").lstrhost("/")
                    self.__UploadFileToDataset(filePath, objectName)
                    tmp = {
                        "ref_id": "",
                        "name": fileName,
                        "path": self.filePrefix + "/" + objectName,
                        "size": int(util.getFileSize(filePath)),
                        "md5": util.getFileMd5(filePath),
                        "frame_id":"",
                        "sensor":"",
                        "meta": {},
                        "anotations": {},
                    }
                    syncData.append(tmp)
                    i += 1
                    if self.debug:
                        per = (i * 100) // total
                        showText = filePath + " =====> " + objectName
                        process = "\r[UPLOAD_FILES][%3s%%]: %s" % (per, showText)
                        print(process)


                    j += 1
                    if j >= 1000:
                        j = 0
                        info = self.dataset.SyncDataToWeb(syncData)
                        if self.debug:
                            print(f"[UPLOAD_FILES] sync data to server, success:{info['succ']}, fail:{info['fail']}")

                        retInfo["succ"] += info["succ"]
                        retInfo["fail"] += info["fail"]
                        syncData = []

        if len(syncData) >= 0:#最后几个没同步的数据
            info = self.dataset.SyncDataToWeb(syncData)
            if self.debug:
                print(f"[UPLOAD_FILES] sync data to server, success:{info['succ']}, fail:{info['fail']}")
            retInfo["succ"] += info["succ"]
            retInfo["fail"] += info["fail"]

        return retInfo

    def UploadFileToDataset(self, filePath):
        '''
        Upload one file to dataset service directly
        :param filePath: local file path
        :return:
        '''
        pathInfo = os.path.split(filePath)
        parentDir = os.path.abspath(os.path.join(pathInfo[0], ".."))
        objectName = pathInfo[0].replace(parentDir, "").strip("/").strip("\\") + "/" + pathInfo[1]
        self.__UploadFileToDataset(filePath, objectName)

        if self.debug:
            print(f"[UPLOAD_ONE_FILE][{filePath}  =====> {objectName}]")

        syncData = {
            "ref_id": "",
            "name": os.path.basename(filePath),
            "path": self.filePrefix+ "/" + objectName,
            "size": int(util.getFileSize(filePath)),
            "md5": util.getFileMd5(filePath),
            "frame_id":"",
            "sensor":"",
            "meta": {},
            "anotations": {},
        }

        info = self.dataset.SyncDataToWeb([syncData])
        if self.debug:
            print(f"[UPLOAD_ONE_FILE] sync data to server, success:{info['succ']}, fail:{info['fail']}")

        return info

    def __UploadFileToDataset(self, filePath, objectName, overlay=False):
        """
        Upload one file to OSS
        :param filePath: local file path
        :param objectName: OSS object name
        :param overlay: overlay upload or not
        :return:
        """
        if not self.datasetId:
            self.RiseException()

        if overlay:
            return self.dataset.PutFileToDataset(objectName, filePath)

        db = UploadDb(self.datasetId)

        if db.findByFilePath(filePath, objectName):
            if self.debug:
                print(f"[UPLOAD_FILE]file: {filePath} already upload")
            db.close()
            return

        db.insertVal(filePath, objectName)
        db.close()

        try:
            res = self.dataset.PutFileToDataset(objectName, filePath)
        except:
            retry = 0
            while True:
                try:
                    res = self.dataset.PutFileToDataset(objectName, filePath)
                    break
                except Exception as e:
                    # print(e)
                    retry += 1
                    if self.debug:
                        print(f"[UPLOAD_FILES][{self.commitId}]Internet error, retry after 5s, attempts: {retry}")
                    time.sleep(5)
                    continue


        return res

    def AddFile(self, filepath, objectName="", referId="", metaData={}, sensor="", frameId="") -> File:
        """
        Create a File object which is used for visualize data
        :param filepath: local file path
        :param objectName: OSS object name
        :param referId: referId
        :param metaData: metaData
        :param sensor: sensor
        :param frameId: frameId
        :return:
        """
        if not self.datasetId:
            self.RiseException()

        if self.dataset.hostingMethod == self.dataset.HOSTING_METHOD_OWN_STORAGE:
            raise Exception(f"own hosting type dataset can not add files, you need use AddUrlFile method instead!")

        if not filepath:
            raise Exception(f"filepath must be set")

        objectName = objectName.replace("\\", "/")

        file = File(filepath, objectName.strip("/"), self.filePrefix, self.dataset.endpoint, self.dataset.datasetType)
        self.fileList.append(file)
        if self.debug:
            print(f"[ADD FILE] total:{len(self.fileList)} filepath:{filepath}, objectName:{file.objectPath}")

        if referId != "":
            if not type(referId) is str:
                raise Exception(f"referId must be a str, {type(referId)} gavin")
            file.referId = referId

        if metaData != {}:
            file.metadata = MetaData(metaData)

        if sensor != "":
            if not type(sensor) is str:
                raise Exception(f"sensor must be a str, {type(sensor)} gavin")
            file.sensor = sensor

        if frameId != "":
            if not type(frameId) is str:
                raise Exception(f"frameId must be a str, {type(frameId)} gavin")
            file.frameId = frameId

        return file

    def AddUrlFile(self, url, referId="", metaData={}, sensor="", frameId="", filename="", md5="", filesize=0) -> File:
        """
        Create a UrlFile object which is used for visualize data
        :param url: url of the file you would like to upload
        :param referId: referId
        :param metaData: metaData
        :param sensor: sensor
        :param frameId: frameId
        :param filename: optional, default will be basename of the url
        :param md5: optional, default will be empty sting
        :param filesize: optional, default will be 0
        :return:
        """
        if not self.datasetId:
            self.RiseException()

        if self.dataset.hostingMethod != self.dataset.HOSTING_METHOD_OWN_STORAGE:
            raise Exception(f"cloud-hosting type dataset can not add url files, you need use AddFile method instead!")

        if not url:
            raise Exception(f"url must be set")

        file = UrlFile(url,  self.dataset.datasetType)
        self.fileList.append(file)
        if self.debug:
            print(f"[ADD URL FILE] url:{url}")

        if referId != "":
            if not type(referId) is str:
                raise Exception(f"referId must be a str, {type(referId)} gavin")
            file.referId = referId

        if metaData != {}:
            file.metadata = MetaData(metaData)

        if sensor != "":
            if not type(sensor) is str:
                raise Exception(f"sensor must be a str, {type(sensor)} gavin")
            file.sensor = sensor

        if frameId != "":
            if not type(frameId) is str:
                raise Exception(f"frameId must be a str, {type(frameId)} gavin")
            file.frameId = frameId

        if filename != "":
            if not type(filename) is str:
                raise Exception(f"filename must be a str, {type(filename)} gavin")
            file.filename = filename

        if md5 != "":
            if not type(md5) is str:
                raise Exception(f"md5 must be a str, {type(md5)} gavin")
            file.md5 = md5

        if filesize != "":
            if not type(filesize) is int:
                raise Exception(f"filesize must be a int, {type(filesize)} gavin")
            file.filesize = filesize

        return file

    def Commit(self, commitId = ""):
        """
        commit a dataset data
        :param commitId: if set, this commit will add to the committed data before
        """
        if not self.datasetId:
            self.RiseException()

        db = CommitDb(commitId)
        if self.debug:
            if commitId:
                print(f"[COMMIT FILE] with commitId:{commitId}")
            else:
                print(f"[COMMIT FILE] without commitId!")
            print(f"[COMMIT FILE] begin insert data to local db")

        if len(self.fileList) <= 0:
            raise Exception("you should add 1 file at least")

        db.insertVal(self.fileList)

        if self.debug:
            print(f"[COMMIT FILE] end insert data to local db")

        db.close()

        self.commitFlag = True
        self.commitId = db.commitId
        return db.commitId

    def Upload(self, overlay=False, commitId = ""):
        """
        upload a committed dataset data and sync annotations to the testin-dataser server
        :param overlay: overlay upload to OSS or not
        :param commitId: if set, this commit will be upload and snync
        """
        if not self.datasetId:
            self.RiseException()

        if commitId != "":
            if self.debug:
                print(f"[UPLOAD_VISUALIZE_FILE] upload visualize files to dataset with committed data: {self.commitId}")
            self.commitFlag = True
            self.commitId = commitId

        if not self.commitFlag:
            self.commitId = self.Commit(commitId)
            if self.debug:
                print(f"[UPLOAD_VISUALIZE_FILE] upload visualize files to dataset without commit, auto commit:{self.commitId}")

        if not self.commitId:
            self.commitId = self.Commit(commitId)
            if self.debug:
                print(f"[UPLOAD_VISUALIZE_FILE] upload visualize files to dataset without commit, auto commit:{self.commitId}")

        db = CommitDb(self.commitId)
        allData = db.fetchAll()
        total = len(allData)
        if total <= 0:
            raise Exception("you should add 1 file at least")

        if self.debug:
            print(f"[UPLOAD_FILES]files total count: {total}")

        syncData = []
        i = 0
        j = 0
        retInfo = {'succ': 0, 'fail': 0}
        for data in allData:
            if data["filepath"] != "" and data["objectPath"] != "":
                self.__UploadFileToDataset(data["filepath"], data["objectPath"], overlay=overlay)

            tmp = {
                "ref_id":data["referid"],
                "name":data["filename"],
                "path":data["osspath"],
                "size":data["filesize"],
                "md5":data["md5"],
                "frame_id":data["frame_id"],
                "sensor":data["sensor"],
                "meta":json.loads(data["metadata"]),
                "anotations":json.loads(data["labeldata"]),
            }
            syncData.append(tmp)
            i += 1
            j += 1

            if self.debug:
                per = (j * 100) // total
                if self.dataset.hostingMethod == self.dataset.HOSTING_METHOD_OWN_STORAGE:
                    showText = data["osspath"]
                    process = f"\r[UPLOAD_URL_FILES][{self.commitId}][%3s%%]: %s" % (per, showText)
                else:
                    showText = data['filepath'] + " =====> " + data['objectPath']
                    process = f"\r[UPLOAD_FILES][{self.commitId}][%3s%%]: %s" % (per, showText)
                print(process)

            if i >= 100:
                i = 0
                info = self.dataset.SyncDataToWeb(syncData)
                if self.debug:
                    print(f"[UPLOAD_FILES] sync data to server, success:{info['succ']}, fail:{info['fail']}")
                retInfo["succ"] += info["succ"]
                retInfo["fail"] += info["fail"]
                syncData = []


        if len(syncData) >= 0:#最后几个没同步的数据
            exit()
            info = self.dataset.SyncDataToWeb(syncData)
            if self.debug:
                print(f"[UPLOAD_FILES] sync data to server, success:{info['succ']}, fail:{info['fail']}")
            retInfo["succ"] += info["succ"]
            retInfo["fail"] += info["fail"]

        db.clearCache()

        return retInfo

    def GetData(self, offset=0, limit=1000, metaData={}, label="", sensor=""):
        """
        search data from testin-dataset service by criteria
        :param offset: offset of the search result
        :param limit: numbers of the search result
        :param metaData: which metadata you would like to use to search
        :param label: search by label
        :param sensor: search by sensor
        all above criteria will be juxtaposition
        """
        if limit > 1000:
            raise Exception("limit must less than 1000")

        return self.dataset.GetData(offset, limit, metaData=metaData, label=label, sensor=sensor, debug=self.debug)

    def GetFileAndLabel(self, fid="", referId=""):
        """
        search data from testin-dataset service by fid or ref_id
        :param fid: fid
        :param ref_id: referid you set when you upload data to testin-dataset service
        """
        if fid == "" and referId == "":
            raise Exception("fid or ref_id must be set")

        if fid:
            return self.dataset.GetFileAndLabelByFid(fid, debug=self.debug)

        return self.dataset.GetFileAndLabelByReferid(referId, debug=self.debug)





